==> presigned_url-0.1.0.tar.gz: presigned_url/src/PresignedUrlService.php <==
<?php

declare(strict_types=1);

namespace Drupal\presigned_url;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\consumers\Entity\ConsumerInterface;
use Drupal\consumers\MissingConsumer;
use Drupal\consumers\Negotiator;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\StreamWrapper\StreamWrapperManagerInterface;
use Drupal\Core\Url;
use Drupal\presigned_url\Exception\PresignedUrlException;
use Drupal\presigned_url\Signature\PresignedUrlSignatureGenerator;
use Drupal\presigned_url\Signature\PresignedUrlSignatureInterface;
use Symfony\Component\DependencyInjection\Attribute\Autowire;
use Symfony\Component\HttpFoundation\Request;

/**
 * Service for handling presigned URLs for private files.
 */
class PresignedUrlService implements PresignedUrlSignatureInterface {

  /**
   * The default expiration time for presigned URLs in seconds (15 minutes).
   */
  public const DEFAULT_EXPIRATION = 900;

  /**
   * Constructs a new PresignedUrlService.
   */
  public function __construct(
    protected readonly TimeInterface $time,
    protected readonly FileUrlGeneratorInterface $fileUrlGenerator,
    protected readonly StreamWrapperManagerInterface $streamWrapperManager,
    protected readonly PresignedUrlSignatureGenerator $signatureGenerator,
    #[Autowire(service: 'consumer.negotiator')]
    protected readonly Negotiator $negotiator,
  ) {}

  /**
   * Creates a presigned URL for a private file.
   *
   * @param string $uri
   *   The URI of the private file.
   * @param int $expires
   *   The expiration time in seconds.
   * @param \Drupal\consumers\Entity\ConsumerInterface|null $consumer
   *   The consumer to scope the presigned URL to.
   *   If NULL, the consumer is negotiated from the request.
   *
   * @return string
   *   The presigned URL.
   *
   * @throws \Drupal\presigned_url\Exception\PresignedUrlException
   *   Presigning failed.
   */
  public function createPresignedUrl(string $uri, int $expires = self::DEFAULT_EXPIRATION, ?ConsumerInterface $consumer = NULL): string {
    $scheme = $this->streamWrapperManager->getScheme($uri);

    // Ensure the file is in the private file system.
    if ($scheme !== 'private') {
      throw new PresignedUrlException('The file is not in the private file system.');
    }

    // Ensure the file exists.
    if (!file_exists($uri)) {
      throw new PresignedUrlException('The file does not exist.');
    }

    if (!$consumer) {
      $consumer = $this->negotiator->negotiateFromRequest();

      if (!$consumer) {
        throw new MissingConsumer('Unable to find the default consumer.');
      }
    }

    // Generate the current date.
    $date = gmdate('Ymd\THis\Z', $this->time->getRequestTime());

    // Get absolute URL for the resource.
    $resourceUrl = $this->fileUrlGenerator->generateAbsoluteString($uri);
    if ($resourceUrl === NULL) {
      throw new PresignedUrlException('Unable to generate the absolute URL for the resource.');
    }

    // Create URL object with query parameters.
    $url = Url::fromUri($resourceUrl, [
      'query' => [
        self::QUERY_PARAM_ALGORITHM => 'HMAC-SHA256',
        self::QUERY_PARAM_EXPIRES => $expires,
        self::QUERY_PARAM_DATE => $date,
        self::QUERY_PARAM_CONSUMER => $consumer->getClientId(),
      ],
    ]);

    // Generate signature.
    $signature = $this->signatureGenerator->generateSignature($url->toString());

    // Add signature to URL query params.
    $url->setOption('query', $url->getOption('query') + [
      self::QUERY_PARAM_SIGNATURE => $signature,
    ]);

    return $url->toString();
  }

  /**
   * Validates a presigned URL.
   *
   * @return bool
   *   Whether the URL is valid.
   */
  public function validatePresignedUrl(Request $request): bool {
    $algorithm = $request->query->get(self::QUERY_PARAM_ALGORITHM);
    $expires = $request->query->get(self::QUERY_PARAM_EXPIRES);
    $signature = $request->query->get(self::QUERY_PARAM_SIGNATURE);
    $date = $request->query->get(self::QUERY_PARAM_DATE);
    $consumerId = $request->query->get(self::QUERY_PARAM_CONSUMER);
    $path = $request->getPathInfo();

    // Validate required parameters.
    if (!$expires || !$signature || !$path || !$date || !$consumerId || $algorithm !== 'HMAC-SHA256') {
      return FALSE;
    }

    // Make sure the current consumer matches the one in the URL.
    $consumer = $this->negotiator->negotiateFromRequest($request);
    if (!$consumer || $consumer->getClientId() !== $consumerId) {
      return FALSE;
    }

    // Get expiration date.
    $expirationDate = $this->parseDate($date)?->modify("+{$expires} seconds");
    if (!$expirationDate) {
      return FALSE;
    }

    // Make sure the request is not expired.
    $now = new \DateTime('now', new \DateTimeZone('UTC'));
    if ($now > $expirationDate) {
      return FALSE;
    }

    // Validate the signature.
    $expectedSignature = $this->signatureGenerator->generateSignature($request->getUri());

    return hash_equals($expectedSignature, $signature);
  }

  /**
   * Parses a ISO 8601 date string into a DateTime object.
   */
  protected function parseDate(string $date): ?\DateTime {
    $parsedDate = \DateTime::createFromFormat('Ymd\THis\Z', $date, new \DateTimeZone('UTC'));
    if ($parsedDate === FALSE) {
      return NULL;
    }

    return $parsedDate;
  }

}

==> presigned_url-0.2.0.tar.gz: presigned_url/src/PresignedUrlService.php <==
<?php

declare(strict_types=1);

namespace Drupal\presigned_url;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\StreamWrapper\StreamWrapperManagerInterface;
use Drupal\Core\Url;
use Drupal\presigned_url\Exception\PresignedUrlException;
use Drupal\presigned_url\Signature\PresignedUrlSignatureGenerator;
use Drupal\presigned_url\Signature\PresignedUrlSignatureInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Service for handling presigned URLs for private files.
 */
class PresignedUrlService implements PresignedUrlSignatureInterface {

  /**
   * The default expiration time for presigned URLs in seconds (15 minutes).
   */
  public const DEFAULT_EXPIRATION = 900;

  /**
   * Constructs a new PresignedUrlService.
   */
  public function __construct(
    protected readonly TimeInterface $time,
    protected readonly FileUrlGeneratorInterface $fileUrlGenerator,
    protected readonly StreamWrapperManagerInterface $streamWrapperManager,
    protected readonly PresignedUrlSignatureGenerator $signatureGenerator,
  ) {}

  /**
   * Creates a presigned URL for a private file.
   *
   * @param string $uri
   *   The URI of the private file.
   * @param int $expires
   *   The expiration time in seconds.
   *
   * @return string
   *   The presigned URL.
   *
   * @throws \Drupal\presigned_url\Exception\PresignedUrlException
   *   Presigning failed.
   */
  public function createPresignedUrl(string $uri, int $expires = self::DEFAULT_EXPIRATION): string {
    $scheme = $this->streamWrapperManager->getScheme($uri);

    // Ensure the file is in the private file system.
    if ($scheme !== 'private') {
      throw new PresignedUrlException('The file is not in the private file system.');
    }

    // Ensure the file exists.
    if (!file_exists($uri)) {
      throw new PresignedUrlException('The file does not exist.');
    }

    // Generate the current date.
    $date = gmdate('Ymd\THis\Z', $this->time->getRequestTime());

    // Get absolute URL for the resource.
    $resourceUrl = $this->fileUrlGenerator->generateAbsoluteString($uri);
    if ($resourceUrl === NULL) {
      throw new PresignedUrlException('Unable to generate the absolute URL for the resource.');
    }

    // Create URL object with query parameters.
    $url = Url::fromUri($resourceUrl, [
      'query' => [
        self::QUERY_PARAM_ALGORITHM => 'HMAC-SHA256',
        self::QUERY_PARAM_EXPIRES => $expires,
        self::QUERY_PARAM_DATE => $date,
      ],
    ]);

    // Generate signature.
    $signature = $this->signatureGenerator->generateSignature($url->toString());

    // Add signature to URL query params.
    $url->setOption('query', $url->getOption('query') + [
      self::QUERY_PARAM_SIGNATURE => $signature,
    ]);

    return $url->toString();
  }

  /**
   * Validates a presigned URL.
   *
   * @return bool
   *   Whether the URL is valid.
   */
  public function validatePresignedUrl(Request $request): bool {
    $algorithm = $request->query->get(self::QUERY_PARAM_ALGORITHM);
    $expires = $request->query->get(self::QUERY_PARAM_EXPIRES);
    $signature = $request->query->get(self::QUERY_PARAM_SIGNATURE);
    $date = $request->query->get(self::QUERY_PARAM_DATE);
    $path = $request->getPathInfo();

    // Validate required parameters.
    if (!$expires || !$signature || !$path || !$date || $algorithm !== 'HMAC-SHA256') {
      return FALSE;
    }

    // Get expiration date.
    $expirationDate = $this->parseDate($date)?->modify("+{$expires} seconds");
    if (!$expirationDate) {
      return FALSE;
    }

    // Make sure the request is not expired.
    $now = new \DateTime('now', new \DateTimeZone('UTC'));
    if ($now > $expirationDate) {
      return FALSE;
    }

    // Validate the signature.
    $expectedSignature = $this->signatureGenerator->generateSignature($request->getUri());

    return hash_equals($expectedSignature, $signature);
  }

  /**
   * Parses a ISO 8601 date string into a DateTime object.
   */
  protected function parseDate(string $date): ?\DateTime {
    $parsedDate = \DateTime::createFromFormat('Ymd\THis\Z', $date, new \DateTimeZone('UTC'));
    if ($parsedDate === FALSE) {
      return NULL;
    }

    return $parsedDate;
  }

}
